local agent_helper = require(AGENT_PATH .. "agent_helper")

function PlatformAdapter:GetShareDataFromFile()
	local local_cfg = cjson.decode(UtilEx:readText(UtilEx:getDataPath() .. "sharedata.txt")) or {}
	return local_cfg
end

function PlatformAdapter:SaveShareDataToFile(data)
	if nil == data then return end
	local cfg_text = cjson.encode(data)
	if nil ~= cfg_text then
		UtilEx:writeText(UtilEx:getDataPath() .. "sharedata.txt", cfg_text)
	end
end

function PlatformAdapter:GetShareValueByKey(key)
	return GLOBAL_CONFIG.share_data[key]
end

function PlatformAdapter:SaveShareValue(key, value)
	GLOBAL_CONFIG.share_data[key] = value
	self:SaveShareDataToFile(GLOBAL_CONFIG.share_data)
end

GLOBAL_CONFIG.share_data = PlatformAdapter:GetShareDataFromFile()

AgentAdapter = {
	is_init = false,
	agent_name = "yz3",

	login_callback = nil,
	login_fail_callback = nil,
	auto_login_flag = false,
	session_id = "",
	pay_success_callback = nil,

	webPayUrl = "http://zhifu.100fenlm.cn/pay/toPayModel/205",
	areaId = 14633,

	pay_type_url = "cls.mg05.youyannet.com/yz3/pay_type.php",
	inquery_money_url = "cls.mg05.youyannet.com/yz3/pay_callback.php",
	inquery_money_time_out = 10,				--查询余额超时时间
	inquery_money_frequency = 15,				--查询余额频率
	inquery_money_back_fail_max_times = 4,		--查询返回失败后查询余额最大次数,上线查的次数和刚充完查的次数不同	
	inquery_money_back_fail_times = 0,
	inquery_money_delay_timer = nil,
	is_pay_inquery_ing = false,

	open_id = "",
	mid = "",
}

AgentAdapter.Platform = {
	ePlatform_Weixin = 1,		--微信平台
	ePlatform_QQ = 2,			--手Q平台
}

AgentAdapter.TokenType = {
    eToken_QQ_Access = 1,    	--手Q accesstoken  查询手Q个人、好友、关系链、分享等功能
    eToken_QQ_Pay = 2,          --手Q paytoken  支付相关
    eToken_WX_Access = 3,       --微信 accesstoken 查询微信个人、好友、关系链、分享、支付等
    eToken_WX_Code = 4,         --游戏不需要关心
    eToken_WX_Refresh = 5,      --微信refreshtoken 刷新accesstoken
}

-- 平台ID
function AgentAdapter:GetSpid()
	return "yz3"
end

-- 平台帐号唯一标识
function AgentAdapter:GetOpenId()
	return self.open_id
end

-- 加前辍的平台帐号，全平台唯一
function AgentAdapter:GetPlatName()
	return "yz3_" .. self.open_id
end

--获得游戏名字
function AgentAdapter:GetGameName()
	return Language.Common.GameName[65]
end

--获得logo资源名
function AgentAdapter:GetLogoResName()
	return "agentres/logo.png"
end

--获得loadingbgname
-- function AgentAdapter:GetLoadingbgName()
-- 	return "loading_bg_xyj"
-- end

function AgentAdapter:Init()
	if self.is_init then return end
	self.is_init = true

	local cache_data = Split((AdapterToLua:getInstance():getDataCache("PRVE_ACCOUNT_NAME") or ""), "_")
	if cache_data ~= nil then
		GLOBAL_CONFIG.package_info.config.agent_id = cache_data[1] or "yz3"
		self.open_id = cache_data[2] or ""
	end

	print("===========", self.open_id)
	AgentMs:InitLocalPush()

	PlatformBinder:JsonBind("event_agent_show_msg", function(result)
		SysMsgCtrl.Instance:ErrorRemind(result, true)
		return true
	end)
end

--点击返回登录认为是注销
function AgentAdapter:OnClickBackLogin()
	AgentAdapter:Logout()
end

--当首次进入游戏场景时（即玩家第一次看到游戏地图时）
function AgentAdapter:OnFirstEnterGameScene()
	AgentAdapter:InqueryMoney()
	PlatformBinder:JsonCall("call_agent_request_mid") -----------出包后再打开
end

-----------------------------------------
--登录相关
-----------------------------------------
function AgentAdapter:Login_InQQ(callback, login_fail_callback)
	self:Login(callback, login_fail_callback, AgentAdapter.Platform.ePlatform_QQ)
end

function AgentAdapter:Login_InWX(callback, login_fail_callback)
	Log("=======Login_InWX========")
	self:Login(callback, login_fail_callback, AgentAdapter.Platform.ePlatform_Weixin)
end

function AgentAdapter:SwitchAccount(callback)
	self.login_callback = callback
	agent_helper:Logout()
end

function AgentAdapter:AutoLogin(callback, login_fail_callback)
	self.login_callback = callback
	self.login_fail_callback = login_fail_callback

	local arg_t = {is_auto_login = "true"}
	agent_helper:Login(LUA_CALLBACK(self, self.LoginCallback), cjson.encode(arg_t))
end

function AgentAdapter:Login(callback, login_fail_callback, platform)
	Log("=======Login========")
	self.login_callback = callback
	self.login_fail_callback = login_fail_callback

	local arg_t = {is_auto_login = "false", platform = platform}

	agent_helper:Login(LUA_CALLBACK(self, self.LoginCallback), cjson.encode(arg_t))
end

function AgentAdapter:LoginCallback(result)
	print("AgentAdapter:LoginCallback" .. result)

	self.open_id = agent_helper:GetSessionId()

	if "true" == result and self.open_id ~= "" and self.open_id ~= nil then
		self.login_callback(self:GetPlatName())
		return true
	else

		SysMsgCtrl.Instance:ErrorRemind("登录失败,请再次尝试", true)
		if nil ~= self.login_fail_callback then
			self.login_fail_callback()
		end
		
	end

	return false
end

function AgentAdapter:Logout()
	agent_helper:Logout()
end

function AgentAdapter:LogoutCallback(result)
	print("AgentAdapter:LogoutCallback " .. result)
	-- if "true" == result then
	-- 	AdapterToLua:getInstance():setDataCache("RELOGIN_FLAG", "true")
	-- 	AdapterToLua:restart()
	-- end
	return true
end

-----------------------------------------
--充值相关
-----------------------------------------
function AgentAdapter:Pay(role_id, role_name, amount, server_id, callback)

	local verify_callback = function(url, arg, data, size)	
		local ret_t = cjson.decode(data)

		local pay_type = AgentAdapter:GetPayType(ret_t)

		if pay_type == 0 then
			--腾讯支付
			AgentAdapter:YybPay(role_id, role_name, amount, server_id, callback)
		elseif pay_type == 1 then
			--中青宝网页支付
			AgentAdapter:ZqbPay(role_id, role_name, amount, server_id, callback)
		end
	end
	HttpClient:Request(self.pay_type_url, "", verify_callback)
end

-- 1. 总开关打开
-- 2. 角色等级≥35级 
-- 3. 在每天0点至9点、18点至24点两个时间段
function AgentAdapter:GetPayType(ret_t)
	local role_vo = GameVoManager.Instance:GetMainRoleVo()
	local user_vo = GameVoManager.Instance:GetUserVo()

	if role_vo ~= nil and user_vo ~= nil then

		local user_level = tonumber(role_vo.level) or 0 --角色等级
		local date=tonumber(os.date("%H")) --当前时间(小时)

		Log("user_level:"..user_level)
		Log("date:"..date)
		ret_t.pay_type = "1"
		if ret_t ~= nil and ret_t.pay_type == "0" then
			return 0
		elseif ret_t ~= nil and ret_t.pay_type == "1" then
			if user_level > 5 then
				if date > 15 and date < 18  then
					return 0
				end
				return 1
			end
			return 0
		end
		return 0
	end
end

function AgentAdapter:encodeURI(s)
    s = string.gsub(s, "([^%w%.%- ])", function(c) return string.format("%%%02X", string.byte(c)) end)
    return string.gsub(s, " ", "+")
end


--应用宝支付
function AgentAdapter:YybPay(role_id, role_name, amount, server_id, callback)
	if AgentAdapter.is_pay_inquery_ing then
		SysMsgCtrl.Instance:ErrorRemind("上一次充值还未到帐，请稍候")
		return
	end

	AgentAdapter.pay_success_callback = callback
	local user_vo = GameVoManager.Instance:GetUserVo()
	print("AgentAdapter.pay:", user_vo.real_server_id)
	AgentAdapter:Recharge(amount, "false", user_vo.real_server_id)
end

--中青宝网页支付
function AgentAdapter:ZqbPay(role_id, role_name, amount, server_id, callback)

	local time = os.time()
	local CpOrderId = server_id .. ":" .. self.open_id .. ":" .. time

	local remark = CpOrderId .. "|" .. self.open_id .. "|" .. amount*100 .. "|"

	remark = self:encodeURI(remark)
	
	local real_url = string.format("%s?accounts=%s&areaId=%s&remark=%s&noDispame=%s", self.webPayUrl, 1, self.areaId, remark,1)
	Log("start verify login: url=" .. real_url)
--http://m.zqgame.com/pay/toPayModel/205?accounts=35D431ABD04D502F9C90CC3D079E6339&areaId=14633&remark=14:35D431ABD04D502F9C90CC3D079E6339:1473845755|35D431ABD04D502F9C90CC3D079E6339|100
	--PlatformBinder:JsonCall("call_agent_open_zqbpay", real_url)
	PlatformAdapter.OpenWebView(real_url)

end
--上报
function AgentAdapter:SubmitRoleData(role_id, role_name, role_level, zone_id, zone_name)
	if self.uid == "" then
		return
	end
	local json_obj = {}
	json_obj.role_id = role_id
	json_obj.role_name = role_name
	json_obj.role_level = role_level
	json_obj.zone_id = zone_id
	json_obj.zone_name = zone_name
	PlatformBinder:JsonCall("call_agent_submit_extend_data", cjson.encode(json_obj))
end

--充值。。
function AgentAdapter:Recharge(default_value, is_can_change, server_id)
	default_value = default_value or "0"
	default_value = default_value * 10   --转成元宝 
	is_can_change = is_can_change or "false"
	Log("AgentAdapter:Recharge" .. default_value, server_id)
	local recharge_info_t = {default_value = default_value, is_can_change = is_can_change, server_id = "1"} -- 写死1了，，不区分
	local recharge_info = cjson.encode(recharge_info_t)
	agent_helper:Recharge(recharge_info)
end

function AgentAdapter:RechargeCallback(data)
	data = data or ""
	Log("AgentAdapter:RechargeCallback" .. data)
	local result = cjson.decode(data)
	if nil == result or nil == result.resultCode or nil == result.payState then
		return true
	end 

	if -1 == result.resultCode then
		SysMsgCtrl.Instance:ErrorRemind("支付流程失败")
	end
	if 2 == result.resultCode then
		SysMsgCtrl.Instance:ErrorRemind("用户取消")
	end

	if 0 == result.resultCode then
		SysMsgCtrl.Instance:ErrorRemind("支付流程成功")
		if 1 == result.payState	then
			SysMsgCtrl.Instance:ErrorRemind("用户取消")
		end
		if 2 == result.payState	then
			SysMsgCtrl.Instance:ErrorRemind("支付出错")
		end
		if 0 == result.payState	then
			SysMsgCtrl.Instance:ErrorRemind("支付成功")
			AgentAdapter.inquery_money_back_fail_max_times = 150
			AgentAdapter.is_pay_inquery_ing = true 
			--查询数额
			AgentAdapter:InqueryMoney()

			if AgentAdapter.pay_success_callback then
				AgentAdapter.pay_success_callback()
			end
		end
	end
	return true
end

--查询余额
function AgentAdapter:InqueryMoney()
	AgentAdapter.inquery_money_back_fail_times = 0

	if AgentAdapter.inquery_money_delay_timer then
		GlobalTimerQuest:CancelQuest(AgentAdapter.inquery_money_delay_timer)
		AgentAdapter.inquery_money_delay_timer = nil
	end

	self:DoInqueryMoney()	
end

function AgentAdapter:DoInqueryMoney()
	local pay_info_str = agent_helper:GetPayInfo()
	local pay_info = cjson.decode(pay_info_str) or {}
	--local server_id = GameVoManager.Instance:GetUserVo().plat_server_id
	local server_id = GameVoManager.Instance:GetUserVo().real_server_id

	local url = AgentAdapter.inquery_money_url .. "?openid=%s&openkey=%s&pay_token=%s&pf=%s&pfkey=%s&server_id=%d&session_id=%s&session_type=%s&channeld=%s"
	url = string.format(url, pay_info.openid, pay_info.openkey, pay_info.pay_token, pay_info.pf, pay_info.pfkey, server_id, pay_info.session_id, pay_info.session_type, self:GetSpid())
	Log("AgentAdapter：DoInqueryMoney," .. url)

	local inquery_callback = function(url, arg, data, size)
		if AgentAdapter.inquery_money_delay_timer then 
			GlobalTimerQuest:CancelQuest(AgentAdapter.inquery_money_delay_timer)
			AgentAdapter.inquery_money_delay_timer = nil
		end
		local result = cjson.decode(data)
		Log("AgentAdapter：InqueryMoneyCallback,data=" .. data) 
		local is_need_inquery_again = false
		if result == nil or result.ret == nil or result.msg == nil then
			is_need_inquery_again = true
		else
			if result.ret ~= 0 then
				if AgentAdapter.inquery_money_back_fail_times < AgentAdapter.inquery_money_back_fail_max_times then
					is_need_inquery_again = true
					AgentAdapter.inquery_money_back_fail_times = AgentAdapter.inquery_money_back_fail_times + 1
				end
			else
				AgentAdapter.is_pay_inquery_ing = false
				local uoid = result.billno or ""
				local money = result.money or 0
				AgentMs:StatisticsPay(uoid, money, 0)
			end	
		end
		
		if is_need_inquery_again then
			AgentAdapter.inquery_money_delay_timer = GlobalTimerQuest:AddDelayTimer(LUA_CALLBACK(AgentAdapter, AgentAdapter.DoInqueryMoney), AgentAdapter.inquery_money_frequency)
		end
	end

	HttpClient:Request(url, "", inquery_callback)
	AgentAdapter.inquery_money_delay_timer = GlobalTimerQuest:AddDelayTimer(LUA_CALLBACK(AgentAdapter, AgentAdapter.DoInqueryMoney), AgentAdapter.inquery_money_time_out)
end

-- 是否支持动态下载
function AgentAdapter:IsDynamicDownloadRes()
	return true
end

agent_helper:SetLogoutCallback(LUA_CALLBACK(AgentAdapter, AgentAdapter.LogoutCallback))
agent_helper:SetRechargeCallback(LUA_CALLBACK(AgentAdapter, AgentAdapter.RechargeCallback))

